@@ -7,7 +7,7 @@ AnyEvent::Impl::Qt - AnyEvent adaptor for Qt
    use AnyEvent;
    use Qt;
   
-   Qt::Application \@ARGV; # REQUIRED!
+   my $app = Qt::Application \@ARGV; # REQUIRED!
   
    # this module gets loaded automatically as required
 
@@ -27,34 +27,6 @@ Avoid Qt if you can.
 
 =cut
 
-no warnings;
-use strict;
-
-package AnyEvent::Impl::Qt::Timer;
-
-use Qt;
-use Qt::isa qw(Qt::Timer);
-use Qt::slots cb => [];
-
-# having to go through these contortions just to get a timer event is
-# considered an advantage over other gui toolkits how?
-
-sub NEW {
-   my ($class, $after, $cb) = @_;
-   shift->SUPER::NEW;
-   this->{cb} = $cb;
-   this->connect (this, SIGNAL "timeout()", SLOT "cb()");
-   this->start ($after * 1000, 1);
-}
-
-sub cb {
-   (delete this->{cb})->();
-}
-
-sub DESTROY {
-   $_[0]->stop;
-}
-
 package AnyEvent::Impl::Qt::Io;
 
 use Qt;
@@ -75,33 +47,57 @@ sub cb {
    this->setEnabled (1);
 }
 
+package AnyEvent::Impl::Qt::Timer;
+
+use Qt;
+use Qt::isa qw(Qt::Timer);
+use Qt::slots cb => [];
+
+# having to go through these contortions just to get a timer event is
+# considered an advantage over other gui toolkits how?
+
+sub NEW {
+   my ($class, $after, $interval, $cb) = @_;
+   shift->SUPER::NEW ();
+   this->{interval} = $interval;
+   this->{cb}       = $cb;
+   this->connect (this, SIGNAL "timeout()", SLOT "cb()");
+   this->start ($after, 1);
+}
+
+sub cb {
+   this->start (this->{interval}, 1);
+   (this->{cb})->();
+}
+
 package AnyEvent::Impl::Qt;
 
+no warnings;
+use strict;
+
+use AnyEvent ();
+
 use Qt;
 use AnyEvent::Impl::Qt::Timer;
 use AnyEvent::Impl::Qt::Io;
 
+our $app = Qt::Application \@ARGV; # REQUIRED!
 sub io {
    my ($class, %arg) = @_;
 
-   # cygwin requires the fh mode to be matching, unix doesn't
-   my ($qt, $mode) = $arg{poll} eq "r" ? (Qt::SocketNotifier::Read (), "<")
-                   : $arg{poll} eq "w" ? (Qt::SocketNotifier::Write(), ">")
-                   : Carp::croak "AnyEvent->io requires poll set to either 'r' or 'w'";
-
    # work around these bugs in Qt:
    # - adding a callback might destroy other callbacks
    # - only one callback per fd/poll combination
-   open my $fh2, "$mode&" . fileno $arg{fh}
-      or die "cannot dup() filehandle: $!";
+   my ($fh, $qt) = AnyEvent::_dupfh $arg{poll}, $arg{fh},
+                      Qt::SocketNotifier::Read (), Qt::SocketNotifier::Write();
 
-   AnyEvent::Impl::Qt::Io $fh2, $qt, $arg{cb}
+   AnyEvent::Impl::Qt::Io $fh, $qt, $arg{cb}
 }
 
 sub timer {
    my ($class, %arg) = @_;
    
-   AnyEvent::Impl::Qt::Timer $arg{after}, $arg{cb}
+   AnyEvent::Impl::Qt::Timer $arg{after} * 1000, $arg{interval} * 1000, $arg{cb}
 }
 
 sub one_event {
